#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/stat.h>
#include <errno.h>
#include "../include/generator.h"
#include "../include/solver.h"
#include "../include/output.h"
#include "../include/utils.h"

#define DEFAULT_SIZE 10

void create_puzzle_directory(unsigned int seed) {
    char dirname[20];
    snprintf(dirname, sizeof(dirname), "puzzle_%u", seed);
    
    if (mkdir(dirname, 0777) == -1 && errno != EEXIST) {
        fprintf(stderr, "Error creating directory: %s\n", dirname);
        exit(1);
    }
}

int main(int argc, char *argv[]) {
    unsigned int seed = (unsigned int)time(NULL);
    int size = DEFAULT_SIZE;

    if (argc > 1) {
        seed = (unsigned int)atoi(argv[1]);
    }
    if (argc > 2) {
        size = atoi(argv[2]);
        if (size < 5 || size > 20) {
            fprintf(stderr, "Size must be between 5 and 20\n");
            return 1;
        }
    }

    create_puzzle_directory(seed);
    init_random(seed);

    GameBoard *board = create_board(size, size);
    if (!board) {
        fprintf(stderr, "Failed to create board\n");
        return 1;
    }

    if (!generate_puzzle(board)) {
        fprintf(stderr, "Failed to generate puzzle\n");
        destroy_board(board);
        return 1;
    }

    char filename[50];
    snprintf(filename, sizeof(filename), "puzzle_%u/unsolved.pbm", seed);
    print_unsolved_puzzle(board, filename);

    if (!solve_puzzle(board)) {
        fprintf(stderr, "Generated puzzle is not solvable\n");
        destroy_board(board);
        return 1;
    }

    snprintf(filename, sizeof(filename), "puzzle_%u/solved.pbm", seed);
    print_solved_puzzle(board, filename);

    snprintf(filename, sizeof(filename), "puzzle_%u/solved.txt", seed);
    print_ascii_puzzle(board, filename);

    destroy_board(board);
    
    return 0;
}