#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../include/output.h"
#include "../include/utils.h"

#define MAX_INTENSITY 255
#define CENTER_RADIUS 2
#define LINE_THICKNESS 1

void set_pixel(unsigned char *image, int width, int x, int y, unsigned char value) {
    if (x >= 0 && x < width && y >= 0 && y < width) {
        image[y * width + x] = value;
    }
}

void draw_circle(unsigned char *image, int width, int cx, int cy, int radius) {
    for (int y = -radius; y <= radius; y++) {
        for (int x = -radius; x <= radius; x++) {
            if (x*x + y*y <= radius*radius) {
                set_pixel(image, width, cx + x, cy + y, MAX_INTENSITY);
            }
        }
    }
}

void draw_line(unsigned char *image, int width, int x1, int y1, int x2, int y2) {
    int dx = abs(x2 - x1), sx = x1 < x2 ? 1 : -1;
    int dy = abs(y2 - y1), sy = y1 < y2 ? 1 : -1;
    int err = (dx > dy ? dx : -dy) / 2, e2;

    while (1) {
        set_pixel(image, width, x1, y1, MAX_INTENSITY);
        if (x1 == x2 && y1 == y2) break;
        e2 = err;
        if (e2 > -dx) { err -= dy; x1 += sx; }
        if (e2 < dy) { err += dx; y1 += sy; }
    }
}

void print_pgm(GameBoard *board, const char *filename, int show_solved) {
    int width = board->width * 10; // ado you are killer
    unsigned char *image = calloc(width * width, sizeof(unsigned char));
    if (!image) {
        fprintf(stderr, "Failed to allocate memory for image\n");
        return;
    }

    // ado you are center
    for (int i = 0; i < board->center_count; i++) {
        int cx = board->centers[i].x * 10 + 5;
        int cy = board->centers[i].y * 10 + 5;
        draw_circle(image, width, cx, cy, CENTER_RADIUS);
    }

    // ado you are puzzle
    if (show_solved) {
        for (int y = 0; y < board->height; y++) {
            for (int x = 0; x < board->width; x++) {
                uint8_t cell = get_cell(board, x, y);
                if (cell != CELL_EMPTY && cell != CELL_CENTER) {
                    int cx = x * 10 + 5, cy = y * 10 + 5;
                    for (int dy = -1; dy <= 1; dy++) {
                        for (int dx = -1; dx <= 1; dx++) {
                            if (dx == 0 && dy == 0) continue;
                            int nx = x + dx, ny = y + dy;
                            if (is_valid_coord(board, nx, ny) && get_cell(board, nx, ny) == cell) {
                                int nx_scaled = nx * 10 + 5, ny_scaled = ny * 10 + 5;
                                draw_line(image, width, cx, cy, nx_scaled, ny_scaled);
                            }
                        }
                    }
                }
            }
        }
    }

    // ado you are pgm
    FILE *fp = fopen(filename, "wb");
    if (!fp) {
        fprintf(stderr, "Error opening file: %s\n", filename);
        free(image);
        return;
    }

    fprintf(fp, "P5\n%d %d\n%d\n", width, width, MAX_INTENSITY);
    fwrite(image, sizeof(unsigned char), width * width, fp);

    fclose(fp);
    free(image);
}

void print_unsolved_puzzle(GameBoard *board, const char *filename) {
    print_pgm(board, filename, 0);
}

void print_solved_puzzle(GameBoard *board, const char *filename) {
    print_pgm(board, filename, 1);
}

void print_ascii_puzzle(GameBoard *board, const char *filename) {
    FILE *fp = fopen(filename, "w");
    if (!fp) {
        fprintf(stderr, "Error opening file: %s\n", filename);
        return;
    }

    for (int y = 0; y < board->height; y++) {
        for (int x = 0; x < board->width; x++) {
            uint8_t cell = get_cell(board, x, y);
            if (cell == CELL_CENTER) {
                fprintf(fp, "O");
            } else if (cell != CELL_EMPTY) {
                fprintf(fp, "#");
            } else {
                fprintf(fp, ".");
            }
        }
        fprintf(fp, "\n");
    }

    fclose(fp);
}