#include <stdbool.h>
#include <stdlib.h>
#include "../include/solver.h"
#include "../include/utils.h"

#define MAX_ITERATIONS 1000000

typedef struct {
    int x, y;
    int center_index;
} CellAssignment;

static bool is_valid_assignment(GameBoard *board, int x, int y, int center_index) {
    Point center = board->centers[center_index];
    int dx = x - center.x;
    int dy = y - center.y;
    
    int sym_x = center.x - dx;
    int sym_y = center.y - dy;
    
    if (sym_x < 0 || sym_x >= board->width || sym_y < 0 || sym_y >= board->height) {
        return false;
    }
    
    uint8_t sym_cell = get_cell(board, sym_x, sym_y);
    return sym_cell == CELL_EMPTY || sym_cell == (center_index + 1);
}

static bool backtrack(GameBoard *board, CellAssignment *stack, int stack_size) {
    if (stack_size == board->width * board->height) {
        return true;
    }
    
    int x = stack_size % board->width;
    int y = stack_size / board->width;
    
    if (get_cell(board, x, y) != CELL_EMPTY) {
        stack[stack_size].x = x;
        stack[stack_size].y = y;
        stack[stack_size].center_index = get_cell(board, x, y) - 1;
        return backtrack(board, stack, stack_size + 1);
    }
    
    for (int i = 0; i < board->center_count; i++) {
        if (is_valid_assignment(board, x, y, i)) {
            set_cell(board, x, y, i + 1);
            stack[stack_size].x = x;
            stack[stack_size].y = y;
            stack[stack_size].center_index = i;
            
            if (backtrack(board, stack, stack_size + 1)) {
                return true;
            }
            
            set_cell(board, x, y, CELL_EMPTY);
        }
    }
    
    return false;
}

bool solve_puzzle(GameBoard *board) {
    CellAssignment *stack = malloc(board->width * board->height * sizeof(CellAssignment));
    if (!stack) return false;

    bool result = backtrack(board, stack, 0);

    free(stack);
    return result;
}